//
//  NSString+Additions.m
//  Perforce
//
//  Created by Adam Czubernat on 16.05.2013.
//  Copyright (c) 2013 Perforce Software, Inc. All rights reserved.
//

#import "NSString+Additions.h"

@implementation NSString (Additions)

+ (NSString *)stringWithTimeInterval:(NSTimeInterval)timeInterval {
	if (timeInterval == INFINITY)
		return @"∞";
	NSInteger time = (NSInteger)timeInterval;
	return [NSString stringWithFormat:@"%@%@%@",
			time < 3600 ? @"" : [NSString stringWithFormat:@"%02ld:", time/3600],
			time < 60 ? @"" : [NSString stringWithFormat:@"%02ld:", (time/60)%60],
			[NSString stringWithFormat:(time < 60 ? @"%lds" : @"%02ld"), time%60]];
}

+ (NSString *)stringWithByteCount:(NSInteger)bytes {
	NSString *units;
	int decimals = 0;
	CGFloat size = bytes;
	if (size < 1024.0f)
		units = @"bytes";
	else if ((size /= 1024.0f) < 1024.0f)
		units = @"KB";
	else if ((size /= 1024.0f) < 1024.0f)
		units = @"MB", decimals = 1;
	else
		size /= 1024.0f, units = @"GB", decimals = 2;
	
	return [NSString stringWithFormat:@"%.*f %@", decimals, size, units];
}

- (NSString *)stringByRemovingPrefix:(NSString *)prefix {
	return ([self hasPrefix:prefix] ?
			[self substringFromIndex:prefix.length] :
			self);
}

- (NSString *)stringByRemovingSuffix:(NSString *)suffix {
	return ([self hasSuffix:suffix] ?
			[self substringToIndex:self.length - suffix.length] :
			self);
}

- (NSArray *)arrayOfArguments {

	CFStringRef string = (__bridge CFStringRef)self;
	CFIndex length = CFStringGetLength(string);
	
	NSMutableArray *array = [NSMutableArray arrayWithCapacity:length];

	unichar quoteChar;
	CFIndex bufferLength = 0;
	unichar *buffer = malloc(sizeof(unichar) * length);
	BOOL inQuote = NO;

	for (CFIndex i=0; i<length; i++) {
		unichar c = CFStringGetCharacterAtIndex(string, i);
		if (!inQuote && (c==0x0020 || c==0x0009 || c==0x000A || c==0x000D || c==0x0085)) {
			if (bufferLength) {
				CFStringRef argument = CFStringCreateWithCharacters(NULL, buffer, bufferLength);
				[array addObject:CFBridgingRelease(argument)];
				bufferLength = 0;
			}
			continue;
		}
		
		if ((c=='\'' || c=='"') && (!inQuote || c==quoteChar)) {
			inQuote = !inQuote;
			quoteChar = c;
			continue;
		}
		
		buffer[bufferLength++] = c;
	}
	
	if (bufferLength) {
		CFStringRef argument = CFStringCreateWithCharacters(NULL, buffer, bufferLength);
		[array addObject:CFBridgingRelease(argument)];
	}
	
	free(buffer);	
	
	return array;
}

- (CGSize)sizeWithFont:(NSFont *)font {
	return [self sizeWithAttributes:@{ NSFontAttributeName : font }];
}

#pragma mark - Path support

- (NSString *)stringByDeletingPath {
	NSRange range = { 0, self.length-1 };
	range = [self rangeOfString:@"/" options:NSBackwardsSearch range:range];
	NSUInteger loc = range.location;
	if (loc != NSNotFound)
		return [self substringToIndex:loc+1];
	return self;
}

- (NSString *)stringByAppendingPath:(NSString *)path {
	return ([self hasSuffix:@"/"] ?
			[self stringByAppendingString:path] :
			[self stringByAppendingFormat:@"/%@", path]);
}

- (NSString *)stringByRenamingPath:(NSString *)newName {
	NSString *newPath = [[self stringByDeletingPath] stringByAppendingPath:newName];
	return [self hasSuffix:@"/"] ? [newPath stringByAppendingString:@"/"] : newPath;
}

- (NSString *)directoryPath {
	if (self.length)
		return [self hasSuffix:@"/"] ? self : [self stringByAppendingString:@"/"];
	return nil;
}

- (BOOL)isSubpath:(NSString *)base {
	NSString *path = [base directoryPath];
	return [self hasPrefixCaseInsensitive:path];
}

- (NSString *)relativePath:(NSString *)base {
	NSString *path = [base directoryPath];
	if ([self hasPrefixCaseInsensitive:path])
		return [self substringFromIndex:path.length];
	return nil;
}

- (BOOL)hasSpecialCharacters {
    if([self rangeOfString:@"#"].location != NSNotFound) return true;
    if([self rangeOfString:@"*"].location != NSNotFound) return true;
    if([self rangeOfString:@"@"].location != NSNotFound) return true;
    if([self rangeOfString:@"%"].location != NSNotFound) {
        NSUInteger length = [self length];
        NSRange range = NSMakeRange(0, length);
        while(range.location != NSNotFound) {

            range = [self rangeOfString: @"%" options:0 range:range];

            if(range.location != NSNotFound) {

                if(range.location + 2 > length) {
                    return true;
                }

                NSRange subrange = NSMakeRange(range.location, 3);
                NSString *code = [self substringWithRange:subrange];

                if(!(

                     [code isEqualToString:@"%25"] ||
                     [code isEqualToString:@"%23"] ||
                     [code isEqualToString:@"%2A"] ||
                     [code isEqualToString:@"%40"]
                     )) {
                    return true;
                }
                
                range = NSMakeRange(range.location + range.length, length - (range.location + range.length));
            }
        }
    }
    return false;
}

- (BOOL)hasEncodedCharacters {

    if(([self rangeOfString:@"%25"].location != NSNotFound) ||
       ([self rangeOfString:@"%23"].location != NSNotFound) ||
       ([self rangeOfString:@"%2A"].location != NSNotFound) ||
       ([self rangeOfString:@"%2a"].location != NSNotFound) ||
       ([self rangeOfString:@"%40"].location != NSNotFound)) {

        return true;

    } else {

        return false;
    }
}

- (NSString *)encodePath {
    // force decoding to prevent double encoding
    NSString *newString = [self decodePath];
    
    newString = [newString stringByReplacingOccurrencesOfString:@"%" withString:@"%25"];
    newString = [newString stringByReplacingOccurrencesOfString:@"#" withString:@"%23"];
    newString = [newString stringByReplacingOccurrencesOfString:@"*" withString:@"%2A"];
    newString = [newString stringByReplacingOccurrencesOfString:@"@" withString:@"%40"];

    return newString;
}

- (NSString *)encodeLucene {
    NSString *newString;
    
    newString = [self stringByReplacingOccurrencesOfString:@"\\" withString:@"\\\\"];
    newString = [newString stringByReplacingOccurrencesOfString:@"+" withString:@"\\+"];
    newString = [newString stringByReplacingOccurrencesOfString:@"-" withString:@"\\-"];
    newString = [newString stringByReplacingOccurrencesOfString:@"&&" withString:@"\\&&"];
    newString = [newString stringByReplacingOccurrencesOfString:@"||" withString:@"\\||"];
    newString = [newString stringByReplacingOccurrencesOfString:@"!" withString:@"\\!"];
    newString = [newString stringByReplacingOccurrencesOfString:@"(" withString:@"\\("];
    newString = [newString stringByReplacingOccurrencesOfString:@")" withString:@"\\)"];
    newString = [newString stringByReplacingOccurrencesOfString:@"{" withString:@"\\{"];
    newString = [newString stringByReplacingOccurrencesOfString:@"}" withString:@"\\}"];
    newString = [newString stringByReplacingOccurrencesOfString:@"[" withString:@"\\["];
    newString = [newString stringByReplacingOccurrencesOfString:@"]" withString:@"\\]"];
    newString = [newString stringByReplacingOccurrencesOfString:@"^" withString:@"\\^"];
    newString = [newString stringByReplacingOccurrencesOfString:@"\"" withString:@"\\\""];
    newString = [newString stringByReplacingOccurrencesOfString:@"~" withString:@"\\~"];
    newString = [newString stringByReplacingOccurrencesOfString:@"*" withString:@"\\*"];
    newString = [newString stringByReplacingOccurrencesOfString:@"?" withString:@"\\?"];
    newString = [newString stringByReplacingOccurrencesOfString:@":" withString:@"\\:"];
    
    return newString;
}


- (NSString *)decodePath {
    NSString *newString;
    newString = [self stringByReplacingOccurrencesOfString:@"%23" withString:@"#"];
    newString = [newString stringByReplacingOccurrencesOfString:@"%2A" withString:@"*"];
    newString = [newString stringByReplacingOccurrencesOfString:@"%2a" withString:@"*"];
    newString = [newString stringByReplacingOccurrencesOfString:@"%40" withString:@"@"];
    newString = [newString stringByReplacingOccurrencesOfString:@"%25" withString:@"%"];

    return newString;
}

#pragma mark -

- (BOOL)isEqualCaseInsensitive:(NSString *)string {
	return string && [self caseInsensitiveCompare:string] == NSOrderedSame;
}

- (BOOL)isEmptyString {
	NSCharacterSet *set = [NSCharacterSet whitespaceAndNewlineCharacterSet];
	return [self stringByTrimmingCharactersInSet:set].length == 0;
}

- (BOOL)hasPrefixCaseInsensitive:(NSString *)prefix {
	return prefix && [self
					  rangeOfString:prefix
					  options:NSAnchoredSearch |
					  NSCaseInsensitiveSearch
					  ].location != NSNotFound;
}

- (BOOL)hasSuffixCaseInsensitive:(NSString *)suffix {
	return suffix && [self
					  rangeOfString:suffix
					  options:NSAnchoredSearch |
					  NSCaseInsensitiveSearch |
					  NSBackwardsSearch
					  ].location != NSNotFound;
}

@end
